// -----------------------------------------------
// OOP - Hausaufgabe 7 - Magisches Quadrat
// Name: _____
// -----------------------------------------------

// Fuer Eingaben:
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

// Fuer Messung der CPU-Zeit:
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;

// Klasse zur Erzeugung magischer Quadrate:
class Quadrat {
	// Anzahl gefundener Quadrate:
	private static int numSquares;

	// Maximalzahl zu druckender Quadrate (darueber nur zaehlen):
	private static final int MAX_PRINT = 20;

	// Pruefung, ob Loesung:
	private static boolean isMagicSquare(int n, int[] square) {
		// Array pruefen:
		if(square == null) {
			System.err.println("Array ist null");
			System.exit(3);
		}
		if(square.length != n*n) {
			System.err.println("Array-Groesse stimmt nicht");
			System.exit(3);
		}

		// Pruefen, dass alle Eintraege zwischen 1 und n^2:
		for(int i = 0; i < square.length; i++) {
			if(square[i] <= 0 || square[i] > n*n) {
				System.err.println("Falscher Wert " +
					square[i] + " an Position " + i);
				System.exit(3);
			}
		}

		// Richtiges Ergebnis fuer Summen berechnen:
		int s = n * (n*n + 1) / 2;

		// Zeilensummen pruefen:
		for(int i = 0; i < square.length; i = i + n) {
			if(sum(n, square, i, 1) != s)
				return false;
		}

		// Spaltensummen pruefen:
		for(int i = 0; i < n; i = i + 1) {
			if(sum(n, square, i, n) != s)
				return false;
		}

		// Diagonale von links oben nach rechts unten:
		if(sum(n, square, 0, n+1) != s)
			return false;

		// Diagonale von rechts oben nach links unten:
		if(sum(n, square, n-1, n-1) != s)
			return false;

		// Testen, dass alle Eintraege verschieden:
		for(int i = 0; i < square.length; i++) {
			for(int j = i + 1; j < square.length; j++) {
				if(square[i] == square[j])
					return false;
			}
		}

		// Alles ok:
		return true;
	}

	// Hilfsfunktion: Summe berechnen
	private static int sum(int n, int[] square, int start, int dist) {
		int s = 0;
		int pos = start;
		for(int i = 0; i < n; i++) {
			s = s + square[pos];
			pos = pos + dist;
		}
		return s;
	}

	// Magisches Quadrat drucken:
	private static void printSquare(int n, int[] square) {
		// Anzahl erhoehen:
		numSquares++;

		// Falls Maximalzahl ueberschritten: Fertig.
		if(numSquares > MAX_PRINT)
			return;

		// Sonst Quadrat drucken:
		for(int i = 0; i < n; i++) {
			for(int j = 0; j < n; j++)
				System.out.format("%2d ", square[i*n+j]);
			System.out.println();
		}
		// Leerzeile zur Trennung mehrerer Quadrate:
		System.out.println();
	}

	// Funktion zur Generierung der magischen Quadrate:
	// ...

	// Hilfsmethode zur Eingabe der Kantenlaenge:
	private static String inputLine(String prompt) {
		// Zur Abwechselung mit einem Buffered Reader:
		BufferedReader in = new BufferedReader(
				new InputStreamReader(System.in));
		System.out.print(prompt + ": ");
		String line = "";
		try {
			line = in.readLine();
		} catch(IOException e) {
			System.err.print("Fehler bei der Eingabe: ");
			System.err.println(e);
			System.exit(2);
		}
		return line;
	}

	// Hauptprogramm:
	public static void main(String[] args) {

		// Argumente aus Kommandozeile pruefen:
		if(args.length > 1) {
			System.err.println("Zu viele Argumente " +
				"in der Kommandozeile.");
			System.exit(2);
		}

		// Eingabe beschaffen (Kommandozeile oder Eingabe):
		String input;
		if(args.length == 1)
			input = args[0];
		else
			input = inputLine("Bitte n eingeben");

		// Eingabe in Zahl umwandeln:
		int n = 3;
		try {
			n = Integer.parseInt(input);
		}
		catch(NumberFormatException e) {
			System.err.println("Argument muss " +
				"eine ganze Zahl sein.");
			System.exit(2);
		}

		// n muss positiv sein:
		if(n <= 0) {
			System.err.println("Fehler: n is negativ.");
			System.exit(2);
		}

		// Beginn der Messung der Realzeit:
		long startTime = System.currentTimeMillis();

		// Magische Quadrate generieren:
		genMagicSquares(n);

		// Verbrauchte CPU-Zeit ausgeben:
		ThreadMXBean tMan = ManagementFactory.getThreadMXBean();
		if(tMan.isCurrentThreadCpuTimeSupported() &&
				tMan.isThreadCpuTimeEnabled()) {
			long cpuTime = tMan.getCurrentThreadCpuTime();
			if(cpuTime != -1) {
				System.out.println(
					"Verbrauchte CPU-Zeit: " +
					(cpuTime / 1000000) + "ms");
			}
		}

		// Abgelaufene Realzeit ausgeben:
		long realTime = System.currentTimeMillis() - startTime;
		if(realTime >= 0) {
			System.out.println("Abgelaufene Realzeit: " +
					realTime + "ms");
		}

		// Anzahl gefundener Quadrat ausgeben:
		System.out.println("Anzahl gefundener Quadrate: " +
					numSquares);
	}
}
